<?php

/**
 * SmartHome-Utils  extension for Class SmartHome,  see:   https://github.com/Bubelbub/SmartHome-PHP
 *
 * @author     michaelano user from http://www.rwe-smarthome-forum.de
 * @version    0.02
 */

namespace michaelano\SmartHomePHP;




//--------------------------------------------------------------------//
// DEFINE CONSTANTS
//--------------------------------------------------------------------//
define('NAGIOS_STATE_OK', 	 0);
define('NAGIOS_STATE_WARNING', 	 1);
define('NAGIOS_STATE_CRITICAL',  2);
define('NAGIOS_STATE_UNKNOWN', 	 3);
// ??? define('NAGIOS_STATE_DEPENDENT', 4);






/**
 * NagiosMsgAndExit()
 *
 * prints plugin output
 *
 * prints formated plugin output and terminates execution. 
 * This should be the last method your plugin calls.
 *
 * @param		string 		$strMessage		text output to be returned to Nagios
 * @param		int 		$intStatus		return code, must be one of the 
 * 											defined STATE_* constants
 *
 * @return  	void (terminates execution)
 */
function NagiosMsgAndExit($intStatus, $strMessage)
{
  switch ($intStatus) 
    {
    case NAGIOS_STATE_OK:
      print "OK - ";
      break;

    case NAGIOS_STATE_WARNING:
      print "WARNING - ";
      break;

    case NAGIOS_STATE_CRITICAL:
      print "CRITICAL - ";
      break;

    case NAGIOS_STATE_UNKNOWN:
      print "UNKNOWN - ";
      break;

    default:
      print "UNKNOWN - wrong plugin exit code";
      exit(NAGIOS_STATE_UNKNOWN);
    }   
  print $strMessage;
  exit($intStatus);
}








/**
 * Get options from the command line or web request
 *
 * @param string $options
 * @param array $longopts
 * @return array
 */
function getoptreq ($options, $longopts)
{
   if (PHP_SAPI === 'cli' || empty($_SERVER['REMOTE_ADDR']))  // command line
   {
      return getopt($options, $longopts);
   }
   else if (isset($_REQUEST))  // web script
   {
      $found = array();

      $shortopts = preg_split('@([a-z0-9][:]{0,2})@i', $options, 0, PREG_SPLIT_DELIM_CAPTURE | PREG_SPLIT_NO_EMPTY);
      $opts = array_merge($shortopts, $longopts);

      foreach ($opts as $opt)
      {
         if (substr($opt, -2) === '::')  // optional
         {
            $key = substr($opt, 0, -2);

            if (isset($_REQUEST[$key]) && !empty($_REQUEST[$key]))
               $found[$key] = $_REQUEST[$key];
            else if (isset($_REQUEST[$key]))
               $found[$key] = false;
         }
         else if (substr($opt, -1) === ':')  // required value
         {
            $key = substr($opt, 0, -1);

            if (isset($_REQUEST[$key]) && !empty($_REQUEST[$key]))
               $found[$key] = $_REQUEST[$key];
         }
         else if (ctype_alnum($opt))  // no value
         {
            if (isset($_REQUEST[$opt]))
               $found[$opt] = false;
         }
      }

      return $found;
   }

   return false;
}







/**
 *  sortArray
 *
 * @param array of arraya $data
 * @param string or integer $field
 */
function sortArray($data, $field) 
  { 
  if (!is_array($field)) 
    $field = array($field); 
  uasort($data, 
      function($a, $b) use($field) 
        { 
        $retval = 0; 
        foreach($field as $fieldname) 
          { 
          if($retval == 0) 
          $retval = strnatcmp($a[$fieldname],$b[$fieldname]); 
          } 
        return $retval; 
        }); 
  return $data; 
} 






function SmartHomeLogin($configIniFile, $hostname = "", $userid = "", $passwd = "")
  {
  if ( ($hostname == "")||($userid == "")||($passwd == "") )
    { // at least one option is not given
    // --- read config.ini (if exists)
    if ( (file_exists($configIniFile)) && (is_readable($configIniFile)) )
      {
      $configIni = null;
      if (($configIni = @parse_ini_file($configIniFile)) === false)
        {
        NagiosMsgAndExit(NAGIOS_STATE_CRITICAL,"error while read config.ini");
        //throw new Exception('[0c802] Error while read configuration file: ' . $configIniFile);
        }
      if ( ($hostname=="")&&(array_key_exists('Host', $configIni)) )
        {
        $hostname = $configIni['Host'];
        }
      if ( ($userid=="")&&(array_key_exists('Username', $configIni)) )
        {
        $userid = $configIni['Username'];
        }
      if ( ($passwd=="")&&(array_key_exists('Password', $configIni)) )
        {
        $passwd = $configIni['Password'];
        }
      }
    else
      {
      NagiosMsgAndExit(NAGIOS_STATE_CRITICAL,"error while access config.ini");
      }
    }
  $sh = new \Bubelbub\SmartHomePHP\SmartHome($hostname,$userid,$passwd);
  $tmp = $sh->getSessionId(); // Seems to be a bug: without this, getConfigVersion() returns nothing 
  return $sh;
  }







function GetConfigR( &$sh, $configcachefile, $debug=false)
{
  // --- read Config cache
  $getconfiguration = true;
  if ( (file_exists($configcachefile)) && (is_readable($configcachefile)) )
    {
    $CONFIGR = unserialize(file_get_contents($configcachefile));
    if ($CONFIGR['Vrs'] == $sh->getConfigVersion() )
      {
      /*if ($debug)
        printf ("Cache valid" . $newLine);*/
      $getconfiguration = false;
      }
    }






  // --- get Configuration
  if ($getconfiguration)
    {
    $configuration = $sh->GetEntities(); 
    if ($debug) 
      {  // show complete configuration
      print_r($configuration); 
      } 

    // --- determine Room infos  
    $LCARR = $configuration->LCs->LC; 
    foreach ($LCARR as $LC)  
      {
      //  fails in case of attributes (like $FN)  :   $ID          = strip_tags($LC->Id->asXML()); 
      $ID          = $LC->Id->__toString(); 
      $NAME        = $LC->Name->__toString();  
      $POSITION    = intval($LC->Position->__toString());
      $ROOMS["$ID"] = array ($NAME, $POSITION);
      }
    $ROOMS = sortArray($ROOMS, 1); // sort by position

    // --- determine BD infos  
    $BDARR = $configuration->BDs->BD; 
    foreach ($BDARR as $BD)  
      {    
      $FN         = $BD["FN"]->__toString();                // SerialNo.
      $APPID      = $BD["AppId"]->__toString(); 
      $ID         = $BD->Id->__toString(); 
      $ROOM       = "";// to be filled later (when fetching $LDS)
      $ROOMPOS    = "";// to be filled later (when fetching $LDS)
      if ( isset($BD->AIPs->Ppt["Value"]) )
        {
        $VALUE      = $BD->AIPs->Ppt["Value"]->__toString() ;  // Type
        }
      else
        {  
        $VALUE = "V.";  // Variable
        }
      $BDS["$ID"] = array($VALUE, $FN, $APPID, $ROOM, $ROOMPOS);
      }

    // --- determine Logical Devices
    $LDARR = $configuration->LDs->LD; 
    foreach ($LDARR as $LD)  
      {
      $ID          = $LD->Id->__toString(); 
      $LCID        = $LD["LCID"]->__toString(); 
      $NAME        = $LD["Name"]->__toString(); 
      $BDID        = $LD->BDId->__toString(); 

      $ROOM        = $ROOMS["$LCID"][0];
      $ROOMPOS     = $ROOMS["$LCID"][1];
      $BDTYPE      = ("$BDID" == "") ? "" : $BDS["$BDID"][0];
 
      if ( (isset($BDID)) && ($BDID != "") )
        { // set ROOM info for BD listing
        $BDS["$BDID"][3] = $ROOM;  
        $BDS["$BDID"][4] = $ROOMPOS;
        }

      $LDS["$ID"]  = array($NAME, $LCID, $BDID, $BDTYPE, $ROOM, $ROOMPOS);
      }

    $LDS = sortArray($LDS, 5); // sort by Room-position

    $BDS = sortArray($BDS, 0); // sort by Type
    $BDS = sortArray($BDS, 4); // sort by Room

    if (empty($ROOMS))
      {
      NagiosMsgAndExit(NAGIOS_STATE_CRITICAL,"shc configuration error: cannot determine rooms");
      }
    if (empty($BDS))
      {
      NagiosMsgAndExit(NAGIOS_STATE_CRITICAL,"shc configuration error: cannot determine rooms");
      }
    if (empty($LDS))
      {
      NagiosMsgAndExit(NAGIOS_STATE_CRITICAL,"shc configuration error: cannot determine rooms");
      }


    // ---
    $CONFIGR['Vrs']   = $sh->getConfigVersion();
    $CONFIGR['Rooms'] = $ROOMS;
    $CONFIGR['BDs']   = $BDS;
    $CONFIGR['LDs']   = $LDS;
    // --- save/cache Configuration
    //print_r($CONFIGR);    
    $content = serialize($CONFIGR);
    file_put_contents($configcachefile, $content);
    }


  return $CONFIGR;
  }
  






function GetLogicalDeviceStates( &$sh, &$CONFIGR, $onlyrst, $debug=false)
{
  //$ROOMS   = $CONFIGR['Rooms'];
  //$BDS     = $CONFIGR['BDs']  ;
  $LDS     = $CONFIGR['LDs']  ;

  $devstates = $sh->getAllLogicalDeviceStates();
  if ($debug) 
    {
    print_r($devstates);
    }
  $LDSARR = $devstates->States->LogicalDeviceState; 
  foreach ($LDSARR as $LD)  /*foreach ($LDSARR as $i => $LDS) */
    {
    $LID         = $LD["LID"]->__toString(); 
    $ROOM        = $LDS["$LID"][4];
    $ROOMPOS     = $LDS["$LID"][5];
    $LDNAME      = $LDS["$LID"][0];
    $TEMPERATURE = isset ($LD["Temperature"])      ? $LD["Temperature"]->__toString()      : null;  // Rst: RoomTemperatureSensorState      Sensor Temperature
    $HUMIDITY    = isset ($LD["Humidity"])         ? $LD["Humidity"]->__toString()         : null;  // Rst: RoomHumiditySensorState         Sensor Humidity
    $PTTMP       = isset ($LD["PtTmp"])            ? $LD["PtTmp"]->__toString()            : null;  // Rst: RoomTemperatureActuatorState    Actor: Temperature
    $OPNMD       = isset ($LD["OpnMd"])            ? $LD["OpnMd"]->__toString()            : null;  // Rst: RoomTemperatureActuatorState    Actor: Auto / Manu
    $WRAC        = isset ($LD["WRAc"])             ? $LD["WRAc"]->__toString()             : null;  // Rst: RoomTemperatureActuatorState    ????
    $VALUE       = isset ($LD->Ppts->Ppt["Value"]) ? $LD->Ppts->Ppt["Value"]->__toString() : null;        // GenericDeviceState   
    $ISON        = isset ($LD["IsOn"])             ? $LD["IsOn"]->__toString()             : (isset ($LD->IsOn) ? $LD->IsOn->__toString() : null);         // SwitchActuatorState   
    $ISSMOKE     = isset ($LD->IsSmokeAlarm)       ? $LD->IsSmokeAlarm->__toString()       : null;        //    
    $BDTYPE      = $LDS["$LID"][3];

    if ($onlyrst)
      {
      // does not work, due missing mapping Sensors with BDS  (TODO later) :     if ("$BDTYPE" == "Rst")
      if ( (isset($TEMPERATURE))||(isset($HUMIDITY))||(isset($PTTMP) )  )
        {
        if ( ! (isset($RSTDATA["$ROOM"]) ))
          {
          $RSTDATA["$ROOM"] = array($TEMPERATURE, $HUMIDITY, $PTTMP, $OPNMD, $WRAC, $ROOMPOS);   // Daten der Heizkörperthermostate (Rst)  
          }
        else if (isset($TEMPERATURE) )
          {
          $RSTDATA["$ROOM"][0] = $TEMPERATURE;
          }
        else if (isset($HUMIDITY)) 
          {
          $RSTDATA["$ROOM"][1] = $HUMIDITY;
          }
        else if (isset($PTTMP) )
          {
          $RSTDATA["$ROOM"][2] = $PTTMP;
          $RSTDATA["$ROOM"][3] = $OPNMD;
          $RSTDATA["$ROOM"][4] = $WRAC;  
          }
        }
      }
    else 
      { // Non Rst
      $STATES["$LID"]  = array($ROOM,$ISON, $VALUE ,$ISSMOKE, $LID, $ROOMPOS);
      }
    }

  if ($onlyrst)
    {
    $RSTDATA = sortArray($RSTDATA,5); // sort by ROOM-Position
    }
  else
    {
    $STATES  = sortArray($STATES,5); // sort by ROOM-Position
    }

  return $onlyrst ? $RSTDATA : $STATES;
  }








function GetMessages( &$sh, &$CONFIGR, $onlynew, $debug=false)
{
  //$ROOMS   = $CONFIGR['Rooms'];
  $BDS     = $CONFIGR['BDs']  ;
  //$LDS     = $CONFIGR['LDs']  ;

  $msgs = $sh->getMessageList();
  if ($debug) 
    { 
    print_r($msgs); 
    }
  $MSGARR = $msgs->MessageList->MessageContainer; 
  $countnew = 0;
  foreach ($MSGARR as $MSG)  
    {
    $STATE         = $MSG["State"]->__toString();               
    $TYPE          = $MSG->Message["Type"]->__toString();               
    $TIMESTAMP     = $MSG->Message["TimeStamp"]->__toString();   

    if (isset($MSG->Message->Parameters->MessageParameter["Key"]) )
      $PARAMKEY      = $MSG->Message->Parameters->MessageParameter["Key"]->__toString();
    else 
      $PARAMKEY      = "";
    if (isset($MSG->Message->Parameters->MessageParameter["Value"]) )
      $PARAMVALUE    = $MSG->Message->Parameters->MessageParameter["Value"]->__toString();
    else 
      $PARAMVALUE    = "";

    $ROOM          = "";
    $BDTYPE        = "";

    if ($PARAMKEY == "DeviceId") 
      {
      $ROOM        = $BDS["$PARAMVALUE"][3];
      $BDTYPE      = $BDS["$PARAMVALUE"][0];
      }

    if ( ("$STATE" == "New") || ( ! $onlynew) )
      {
      $MSGS["$TIMESTAMP"] = array($STATE, $TYPE, $PARAMKEY, $PARAMVALUE, $ROOM, $BDTYPE);            
      }
    }

  if ( ! empty($MSGS) )
    {
    asort($MSGS);
    $MSGS = array_reverse($MSGS, true);
    }
  else
    {
    $MSGS = null;
    }

  return $MSGS;
}
?>
